/*Copyright (C) 2019 Centro de Computacao Cientifica e Software Livre
Departamento de Informatica - Universidade Federal do Parana

This file is part of Plataforma Integrada MEC.

Plataforma Integrada MEC is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Plataforma Integrada MEC is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with Plataforma Integrada MEC.  If not, see <http://www.gnu.org/licenses/>.*/

import React, {useState} from 'react'
import styled from 'styled-components'
import Button from '@material-ui/core/Button';
import ModalConfirmarUnfollow from '../ModalConfirmarUnfollow.js'
import {putRequest} from '../HelperFunctions/getAxiosConfig'

//Image Import
import { FollowingIcon } from "ImportImages.js";

export default function FollowingButton (props) {

    const [modalOpen, toggleModal] = useState(false)
    const [unfollowID, setUnfollowID] = useState(-1)

    const [followingHover, handleFollowingHover] = useState(false)
    const toggleFollowingHover = (value) => {handleFollowingHover(value)}

    const handleUnfollowPartOne = (followedID) => {
        setUnfollowID(followedID)
        toggleModal(true)
    }


    function handleSuccess (data) {
        props.toggleFollowed()
        toggleModal(false)
    }
    const handleUnfollowPartTwo = () => {
        const url = `/users/${unfollowID}/follow/`

        putRequest(url, {}, handleSuccess, (error) => {console.log(error)})
    }

    return (
        <React.Fragment>
            <ModalConfirmarUnfollow open={modalOpen}
                handleClose={() => {toggleModal(false)}}
                handleConfirm = {handleUnfollowPartTwo}
            />
            <StyledButton
                onMouseOver={() => toggleFollowingHover(true)}
                onMouseLeave={() => toggleFollowingHover(false)}
                onClick={() => handleUnfollowPartOne(props.followedID)}
            >
                {
                    followingHover?
                    (
                        [
                            <span>DEIXAR DE SEGUIR</span>
                        ]
                    )
                    : (
                        [
                            <>
                            <img src={FollowingIcon} alt='ícone seguindo'/><span>Seguindo</span>
                            </>
                    ]
                )
            }
            </StyledButton>
        </React.Fragment>
    )
}

export function NoIconFollowing (props) {
    const [modalOpen, toggleModal] = useState(false)
    const [unfollowID, setUnfollowID] = useState(-1)

    const [followingHover, handleFollowingHover] = useState(false)
    const toggleFollowingHover = (value) => {handleFollowingHover(value)}

    const handleUnfollowPartOne = (followedID) => {
        setUnfollowID(followedID)
        toggleModal(true)
    }


    function handleSuccess (data) {
        props.toggleFollowed()
        toggleModal(false)
    }
    const handleUnfollowPartTwo = () => {
        const url = `/users/${unfollowID}/follow/`

        putRequest(url, {}, handleSuccess, (error) => {console.log(error)})
    }

    return (
        <React.Fragment>
            <ModalConfirmarUnfollow open={modalOpen}
                handleClose={() => {toggleModal(false)}}
                handleConfirm = {handleUnfollowPartTwo}
            />
            <NoIconButton
                onMouseOver={() => toggleFollowingHover(true)}
                onMouseLeave={() => toggleFollowingHover(false)}
                onClick={() => handleUnfollowPartOne(props.followedID)}
            >
                {
                    followingHover?
                    (
                        [
                            <span>DEIXAR DE SEGUIR</span>
                        ]
                    )
                    : (
                        [
                            <span>SEGUINDO</span>
                    ]
                )
            }
            </NoIconButton>
        </React.Fragment>
    )
}


export const StyledButton = styled(Button)`
    background-color : #00bcd4 !important;
    color : #fff !important;
    display : inline-block !important;
    position : relative !important;
    cursor : pointer !important;
    min-height : 36px !important;
    min-width : 88px !important;
    line-height : 36px !important;
    vertical-align : middle !important;
    -webkit-box-align : center !important;
    -webkit-align-items : center !important;
    -ms-grid-row-align : center !important;
    align-items : center !important;
    text-align : center !important;
    border-radius : 3px !important;
    -webkit-user-select : none !important;
    -moz-user-select : none !important;
    -ms-user-select : none !important;
    user-select : none !important;
    border : 0 !important;
    padding : 0 6px !important;
    margin : 6px 8px !important;
    white-space : nowrap !important;
    text-transform : uppercase !important;
    font-weight : 500 !important;
    font-size : 14px !important;
    font-style : inherit !important;
    font-variant : inherit !important;
    font-family : inherit !important;
    text-decoration : none !important;
    overflow : hidden !important;
    &:hover {
        background-color : #fff !important;
        color : #00bcd4 !important;
        border : 1px solid #00bcd4 !important;
    }
    img {
        height : 24px;
        display : inline-block;
        vertical-align : middle;
        color : #fff !important;
    }
`

const NoIconButton = styled(Button)`
    .MuiButton-label {
        color : #00bcd4 !important;
    }
    background-color : #fff !important;
    border : solid 1px rgba(0,188,212,.85) !important;
    font-size : 14px !important;
    border-radius : 3px !important;
    height : 36px !important;
    min-width : 88px !important;
    vertical-align : middle !important;
    text-color : #00bcd4 !important;
    align-items : center !important;
    text-align : center !important;
    padding : 0 6px !important;
    margin : 6px 8px !important;
`
